@@ -69,6 +69,7 @@ struct _GnomeCanvasRichTextExtPrivate {
 	int left_margin;
 	int right_margin;
 	int indent;
+	GtkTextDirection text_direction;
 
 	guint preblink_timeout;
 	guint blink_timeout;
@@ -106,6 +107,7 @@ enum {
 	PROP_LEFT_MARGIN,
 	PROP_RIGHT_MARGIN,
 	PROP_INDENT,
+	PROP_TEXT_DIRECTION,
 };
 
 enum {
@@ -149,7 +151,7 @@ static void gnome_canvas_rich_text_ext_move_cursor(GnomeCanvasRichTextExt *text,
 					       gint count,
 					       gboolean extend_selection);
 static void   gnome_canvas_rich_text_ext_print       (GPrintable *gprintable, GnomePrintContext *pc);
-
+static void gnome_canvas_rich_text_set_cursor_direction (GnomeCanvasRichTextExt *text);
 
 static GtkTextBuffer *get_buffer(GnomeCanvasRichTextExt *text);
 static gint blink_cb(gpointer data);
@@ -393,6 +395,15 @@ gnome_canvas_rich_text_ext_class_init(GnomeCanvasRichTextExtClass *klass)
 				  G_MININT, G_MAXINT,
 				  0,
 				  G_PARAM_READWRITE));
+	g_object_class_install_property (
+		gobject_class,
+		PROP_TEXT_DIRECTION,
+		g_param_spec_int ("text_direction",
+				  _("Text direction"),
+				  _("Text direction (rtl, ltr or default)"),
+				  GTK_TEXT_DIR_NONE, GTK_TEXT_DIR_RTL,
+				  GTK_TEXT_DIR_NONE,
+				  G_PARAM_READWRITE));
 
 	/* Signals */
 	signals[TAG_CHANGED] = g_signal_new(
@@ -437,6 +448,7 @@ gnome_canvas_rich_text_ext_init(GnomeCanvasRichTextExt *text)
 	text->_priv->justification = GTK_JUSTIFY_LEFT;
 	text->_priv->direction = gtk_widget_get_default_direction();
 	text->_priv->anchor = GTK_ANCHOR_NW;
+	text->_priv->text_direction = GTK_TEXT_DIR_NONE;
 	
 	text->_priv->blink_timeout = 0;
 	text->_priv->preblink_timeout = 0;
@@ -607,6 +619,11 @@ gnome_canvas_rich_text_ext_set_property (GObject *object, guint property_id,
 			gtk_text_layout_default_style_changed(text->_priv->layout);
 		}
 		break;
+	case PROP_TEXT_DIRECTION:
+		text->_priv->text_direction = (GtkTextDirection) g_value_get_int (value);
+		if (text->_priv->layout)
+			gnome_canvas_rich_text_set_cursor_direction (text);
+		break;
 		       
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -674,6 +691,8 @@ gnome_canvas_rich_text_ext_get_property (GObject *object, guint property_id,
 	case PROP_INDENT:
 		g_value_set_int (value, text->_priv->indent);
 		break;
+	case PROP_TEXT_DIRECTION:
+		g_value_set_int (value, (int) text->_priv->text_direction);
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
@@ -1842,6 +1861,40 @@ gnome_canvas_rich_text_ext_set_attributes_from_style(GnomeCanvasRichTextExt *tex
 
 } /* gnome_canvas_rich_text_ext_set_attributes_from_style */
 
+static void
+gnome_canvas_rich_text_set_cursor_direction (GnomeCanvasRichTextExt *text)
+{
+	GtkTextDirection new_cursor_dir;
+	GtkTextDirection new_keyboard_dir;
+	gboolean split_cursor;
+	GtkWidget *canvas;
+	GtkSettings *settings; 
+	GdkKeymap *keymap;
+
+	canvas = GTK_WIDGET(GNOME_CANVAS_ITEM(text)->canvas);
+	settings = gtk_widget_get_settings (canvas);
+	g_object_get (settings,
+		"gtk-split-cursor", &split_cursor,
+		NULL);
+
+	if (text->_priv->text_direction == GTK_TEXT_DIR_NONE) {
+		keymap = gdk_keymap_get_for_display (gtk_widget_get_display (canvas));
+		if (gdk_keymap_get_direction (keymap) == PANGO_DIRECTION_LTR)
+			new_keyboard_dir = GTK_TEXT_DIR_LTR;
+		else
+			new_keyboard_dir  = GTK_TEXT_DIR_RTL;
+	} else
+		new_keyboard_dir = text->_priv->text_direction;
+
+	if (split_cursor)
+		new_cursor_dir = GTK_TEXT_DIR_NONE;
+	else
+		new_cursor_dir = new_keyboard_dir;
+
+	gtk_text_layout_set_cursor_direction (text->_priv->layout, new_cursor_dir);
+	gtk_text_layout_set_keyboard_direction (text->_priv->layout, new_keyboard_dir);
+}
+
 static void
 gnome_canvas_rich_text_ext_ensure_layout(GnomeCanvasRichTextExt *text)
 {
@@ -1873,6 +1926,7 @@ gnome_canvas_rich_text_ext_ensure_layout(GnomeCanvasRichTextExt *text)
 		pango_context_set_base_dir(ltr_context, PANGO_DIRECTION_LTR);
 		rtl_context = gtk_widget_create_pango_context(canvas);
 		pango_context_set_base_dir(rtl_context, PANGO_DIRECTION_RTL);
+		gnome_canvas_rich_text_set_cursor_direction (text);
 
 		gtk_text_layout_set_contexts(
 			text->_priv->layout, ltr_context, rtl_context);
