@@ -36,7 +36,11 @@
 
 #include "gcp-canvas-text.h"
 #include "gprintable.h"
-#include "gnome-print-pango.h"
+#ifdef GP_HAS_PANGO
+#	include <libgnomeprint/gnome-print-pango.h>
+#else
+#	include "gnome-print-pango.h"
+#endif
 
 static void gnome_canvas_text_ext_class_init   (GnomeCanvasTextExtClass *class);
 static void gnome_canvas_text_ext_init         (GnomeCanvasTextExt     *line);
