@@ -418,7 +418,7 @@ bool gcpText::OnChanged(GtkTextBuffer *textbuffer)
 	GdkRectangle rect;
 	GtkTextIter iter;
 	gtk_text_buffer_get_start_iter(m_buf,&iter);
-	int  y0 = 0, offset;
+	int  y0 = 0, offset, left = 0;
 	m_height = 1;
 	bool bNotEnd = true;
 	m_length = 0;
@@ -431,6 +431,10 @@ bool gcpText::OnChanged(GtkTextBuffer *textbuffer)
 	}
 	while (bNotEnd)
 	{
+		gtk_text_iter_backward_line(&iter);
+		gnome_canvas_rich_text_ext_get_iter_location(text, &iter, &rect);
+		if (left > rect.x)
+			left = rect.x;
 		bNotEnd = gtk_text_iter_forward_to_line_end(&iter);
 		gnome_canvas_rich_text_ext_get_iter_location(text, &iter, &rect);
 		if (m_length <= rect.x + rect.width) m_length = rect.x + rect.width + 1;
@@ -451,6 +455,7 @@ bool gcpText::OnChanged(GtkTextBuffer *textbuffer)
 			gtk_text_iter_forward_to_line_end(&iter);
 		}
 		offset = gtk_text_iter_get_offset(&iter);
+		m_length -= left;
 		pView->Update(this);
 		while (gtk_events_pending()) gtk_main_iteration();
 		gtk_text_buffer_get_iter_at_offset(m_buf, &iter, offset + 1);
